'''
Created on 18 Jan 2021

@author: jacklok
'''
import unittest, logging
from trexanalytics.util.chart_util import generate_month_before
from datetime import datetime, date
logger = logging.getLogger('unittest')

class TestChartUtil(unittest.TestCase):
    

    def test_generate_month_before(self):
        start_datetime=date(2020,11,30)
        expected_result = generate_month_before(6, start_datetime=start_datetime)
        assert expected_result == [6,7,8,9,10,11]
        
        expected_result = generate_month_before(3, start_datetime=start_datetime)
        assert expected_result == [9,10,11]
        
        expected_result = generate_month_before(1, start_datetime=start_datetime)
        assert expected_result == [11]
        
        expected_result = generate_month_before(0, start_datetime=start_datetime)
        assert expected_result is None
        

if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()



'''
Created on 18 Jan 2021

@author: jacklok
'''
import dateutil.relativedelta
from datetime import datetime
from trexlib.utils.common.common_util import sort_list

def generate_month_before(number_of_months, start_datetime=None):
    if number_of_months and isinstance(number_of_months, int):
        if start_datetime is None:
            start_datetime = datetime.now()
        
            
        
        return sorted([ (start_datetime + dateutil.relativedelta.relativedelta(months=-n)).month for n in range(number_of_months)])



'''
Created on 21 Jan 2021

@author: jacklok
'''
import unittest, logging, sys
from trexlib.utils.google.cloud_tasks_util import create_task

logger = logging.getLogger('unittest')

class TestCryptoUtil(unittest.TestCase):
    
    def test_create_task(self):
        task_url = 'https://trex-analytics-dev.df.r.appspot.com/ping'
        queue_name = 'common'
        create_task(task_url, queue_name, in_seconds=1, http_method = 'GET')
        
        
if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()        



'''
Created on 26 Jan 2021

@author: jacklok
'''

from flask import Blueprint, render_template, session, abort, redirect, url_for, request, jsonify
import logging, json, uuid
from flask.globals import current_app
from trexlib.utils.google.bigquery_util import create_bigquery_client, stream_data_by_datetime_partition, stream_data
from trexanalytics import conf as analytics_conf
from trexanalytics.bigquery_table_template_config import CUSTOMER_TRANSACTION_TEMPLATE, MERCHANT_REGISTERED_CUSTOMER_TEMPLATE
from trexmodel.models.datastore.analytic_models import UpstreamData
from trexmodel.utils.model.model_util import create_db_client
from datetime import datetime
from trexlib.utils.log_util import get_tracelog
from trexanalytics.controllers.importdata.import_data_base_routes import TriggerImportDataBaseResource, InitImportDataBaseResource, ImportDataBaseResource
from flask_restful import Api
from trexlib.utils.string_util import is_not_empty, random_string
from trexanalytics.bigquery_table_template_config import TABLE_SCHEME_TEMPLATE


import_upstream_data_bp = Blueprint('import_upstream_data_bp', __name__,
                     template_folder='templates',
                     static_folder='static',
                     url_prefix='/import-upstream-data')

logger = logging.getLogger('import')

import_upstream_data_api = Api(import_upstream_data_bp)

class TriggerImportAllUpstreamData(TriggerImportDataBaseResource):
    
    def get_task_url(self):
        return '/import-upstream-data/init-import-all-upstream-data'
    
    def get_task_queue(self):
        return 'import-upstream'
    
    
    
    
class InitImportAllUpstreamData(InitImportDataBaseResource):
    def get_import_data_count(self, **kwargs):
        return count_all_unsend_upstream_data()
    
    def get_import_data_page_size(self):
        return analytics_conf.STREAM_DATA_PAGE_SIZE
    
    def get_task_url(self):
        return '/import-upstream-data/import-all-upstream-data'
    
    def get_task_queue(self):
        return 'import-upstream'
    

def count_all_unsend_upstream_data():
    db_client = create_db_client(caller_info="count_all_unsend_upstream_data")
    with db_client.context():
        count       = UpstreamData.count_not_sent()
    return count


class ImportAllUpstreamData(ImportDataBaseResource): 
    def import_data(self, offset, limit, **kwargs):
        logger.debug('Going to import data now')
        logger.debug('offset=%d limit=%d', offset, limit)
        
        
        try:
            start_cursor    = kwargs.get('start_cursor')
            batch_id        = kwargs.get('batch_id')
            
            logger.debug('ImportAllUpstreamData: start_cursor=%s', start_cursor)
            
            next_cursor     = import_all_unsend_upstream_data_function(dataset_name='system', limit=limit, start_cursor=start_cursor, batch_id=batch_id)
            
            logger.debug('ImportAllUpstreamData: next_cursor=%s', next_cursor)
            
            return next_cursor
        except:
            logger.debug('Failed due to %s', get_tracelog())
        
    
    def get_task_url(self):
        return '/import-upstream-data/import-all-upstream-data'
    
    def get_task_queue(self):
        return 'import-upstream'
    
def import_all_unsend_upstream_data_function(credential_info=None, dataset_name='system_dataset', 
                                            limit=analytics_conf.STREAM_DATA_PAGE_SIZE, start_cursor=None, batch_id=None):
    
    bg_client       = create_bigquery_client(credential_filepath=analytics_conf.BIGQUERY_SERVICE_CREDENTIAL_PATH)
    
    if credential_info is None:
        credential_info = current_app.config['credential_config']
        
    db_client = create_db_client(info=credential_info, caller_info="import_registered_customer_function")
    import_upstream_dict = {}
    
    logger.debug('import_all_unsend_upstream_data_function (%s): start_cursor=%s', batch_id, start_cursor)
    
    
    
    with db_client.context():
        (upstream_data_list, next_cursor) = UpstreamData.list_not_send(limit = limit, start_cursor=start_cursor, return_with_cursor=True)
    
    import_count    = len(upstream_data_list)
    errors          = []    
    
    for u in upstream_data_list:
        
        table_template_name = u.table_template_name
        dataset_name        = u.dataset_name
        table_name          = u.table_name
        stream_content      = u.stream_content
        
        import_upstream_dict = {'': stream_content}


    
        logger.debug('################################ batch_id= %s ############################################', batch_id)
        logger.debug(import_upstream_dict)
        logger.debug('#####################################################################################################')
        
        __errors = stream_data(bg_client, dataset_name, table_template_name, 
                                                   TABLE_SCHEME_TEMPLATE.get(table_template_name), table_name, stream_content
                                                   )
        
        if len(__errors)>0:
            errors.extend(__errors)
    
    if errors==[]:
        with db_client.context():
            for u in upstream_data_list:
                u.is_sent = True
                u.put()
                
                
        logger.debug("New rows have been added")
    else:
        logger.debug("Encountered errors while inserting rows: {}".format(errors))
    
    logger.debug('import_count=%d', import_count)
    
    return next_cursor
    
@import_upstream_data_bp.route('/upstream/data-content/<upstream_data_key>')
def read_upstream_data_content(upstream_data_key):
    
    logger.debug('upstream_data_key=%s', upstream_data_key)
    
    try:
        db_client = create_db_client(caller_info="read_upstream_data_content")
        with db_client.context():
            upstream_data = UpstreamData.fetch(upstream_data_key)
            
        if upstream_data:
            return jsonify(upstream_data.stream_content)
    except:
        logger.debug('Failed to fetch upstream data due to %s', get_tracelog())
    
    return 'Not found', 200

import_upstream_data_api.add_resource(TriggerImportAllUpstreamData,   '/trigger-import-all-upstream-data')
import_upstream_data_api.add_resource(InitImportAllUpstreamData,      '/init-import-all-upstream-data')
import_upstream_data_api.add_resource(ImportAllUpstreamData,          '/import-all-upstream-data')

        


'''
Created on 18 Sep 2020

@author: jacklok
'''
import logging, os, config_path

APPLICATION_NAME                                    = 'Trex Analytics'
APPLICATION_TITLE                                   = 'Trex Analytics Platform'
APPLICATION_DESC                                    = 'Trex Analytics is used to anlayze data project'
APPLICATION_HREF                                    = 'http://localhost.com:8084/'

PRODUCTION_MODE                                     = "PROD"
DEMO_MODE                                           = "DEMO"
LOCAL_MODE                                          = "LOCAL"


#DEPLOYMENT_MODE                                     = PRODUCTION_MODE
DEPLOYMENT_MODE                                     = DEMO_MODE
#DEPLOYMENT_MODE                                     = LOCAL_MODE

#DEPLOYMENT_MODE                                     = PRODUCTION_MODE
#DEPLOYMENT_MODE                                     = DEMO_MODE
DEPLOYMENT_MODE                                     = LOCAL_MODE


CSRF_ENABLED                                        = True

CONTENT_WITH_JAVASCRIPT_LINK                        = True
 
APPLICATION_VERSION_NO                              = "25092020"

DEFAULT_LANGUAGE                                    = 'en_us'

GCLOUD_PROJECT_ID                                   = os.environ.get('GCLOUD_PROJECT_ID')
#ANALYTICS_GCLOUD_PROJECT_ID                         = os.environ.get('ANALYTICS_GCLOUD_PROJECT_ID')
#ANALYTICS_GCLOUD_LOCATION                           = os.environ.get('ANALYTICS_GCLOUD_LOCATION')

SUPERUSER_ID                                        = os.environ.get('SUPERUSER_ID')
SUPERUSER_EMAIL                                     = os.environ.get('SUPERUSER_EMAIL')
SUPERUSER_HASHED_PASSWORD                           = os.environ.get('SUPERUSER_HASHED_PASSWORD')

#SERVICE_ACCOUNT_KEY_FILEPATH                        = os.environ['SERVICE_ACCOUNT_KEY']
#SERVICE_CREDENTIAL_PATH                             = os.path.abspath(os.path.dirname(__file__)) + '/../' + SERVICE_ACCOUNT_KEY_FILEPATH

#ANALYTICS_SERVICE_ACCOUNT_KEY_FILEPATH              = os.environ['ANALYTICS_SERVICE_ACCOUNT_KEY']
#ANALYTICS_SERVICE_CREDENTIAL_PATH                   = os.path.abspath(os.path.dirname(__file__)) + '/../' + ANALYTICS_SERVICE_ACCOUNT_KEY_FILEPATH

BIGQUERY_GCLOUD_PROJECT_ID                          = os.environ.get('BIGQUERY_GCLOUD_PROJECT_ID')
BIGQUERY_SERVICE_ACCOUNT_KEY_FILEPATH               = os.environ['BIGQUERY_SERVICE_ACCOUNT_KEY']
BIGQUERY_SERVICE_CREDENTIAL_PATH                    = os.path.abspath(os.path.dirname(config_path.__file__)) +'/'+ BIGQUERY_SERVICE_ACCOUNT_KEY_FILEPATH
BIGQUERY_GCLOUD_LOCATION                            = os.environ.get('BIGQUERY_GCLOUD_LOCATION')

ANALYTICS_BASE_URL                                  = os.environ.get('ANALYTICS_BASE_URL')
UPSTREAM_BASE_URL                                   = os.environ.get('UPSTREAM_BASE_URL')
#BASE_URL                                            = 'http://a66fac3caa6b.ngrok.io'

STREAM_DATA_PAGE_SIZE                               = 100

SYSTEM_DATASET                                      = os.environ.get('SYSTEM_DATASET')
MERCHANT_DATASET                                    = os.environ.get('MERCHANT_DATASET')

ANALYTICS_TASK_GCLOUD_PROJECT_ID                    = os.environ.get('ANALYTICS_TASK_GCLOUD_PROJECT_ID')
ANALYTICS_TASK_GCLOUD_LOCATION                      = os.environ.get('ANALYTICS_TASK_GCLOUD_LOCATION')
ANALYTICS_TASK_SERVICE_ACCOUNT_KEY_FILEPATH         = os.environ.get('ANALYTICS_TASK_SERVICE_ACCOUNT_KEY')
ANALYTICS_TASK_SERVICE_CREDENTIAL_PATH              = os.path.abspath(os.path.dirname(config_path.__file__)) + '/' + ANALYTICS_TASK_SERVICE_ACCOUNT_KEY_FILEPATH
ANALYTICS_TASK_SERVICE_ACCOUNT_EMAIL                = os.environ.get('ANALYTICS_TASK_SERVICE_ACCOUNT_EMAIL')

SERVER_DATETIME_GMT                                 = os.environ.get('SERVER_DATETIME_GMT')


UPSTREAM_UPDATED_DATETIME_FIELD_NAME                = 'UpdatedDateTime'

if DEPLOYMENT_MODE==PRODUCTION_MODE:
    DEBUG_MODE       = False
    #DEBUG_MODE       = True

    #LOGGING_LEVEL    = logging.DEBUG
    #LOGGING_LEVEL    = logging.WARN
    LOGGING_LEVEL    = logging.INFO
    #LOGGING_LEVEL    = logging.ERROR
    
    
    
elif DEPLOYMENT_MODE==DEMO_MODE:
    DEBUG_MODE       = True
    #DEBUG_MODE       = False
    
    LOGGING_LEVEL    = logging.DEBUG
    #LOGGING_LEVEL    = logging.INFO
    
    

elif DEPLOYMENT_MODE==LOCAL_MODE:
    DEBUG_MODE       = True

    LOGGING_LEVEL    = logging.DEBUG
    #LOGGING_LEVEL    = logging.INFO
    





'''
Created on 19 Jan 2021

@author: jacklok
'''

from flask import Blueprint, render_template, request, Response
from flask_restful import Resource
from trexanalytics import conf as analytics_conf
import logging
from trexlib.utils.log_util import get_tracelog

from trexlib.utils.google.bigquery_util import execute_query, process_job_result_into_category_and_count, create_bigquery_client

logger = logging.getLogger('analytic')

class QueryBaseResource(Resource):
    
    def post(self):
        return self.get()
    
    def get(self):
        content         = request.json or request.args.to_dict() or {}
        query           = self.prepare_query(**content)
        
        logger.info('query=%s', query)
        
        bg_client       = create_bigquery_client()
        try:
            job_result_rows = execute_query(bg_client, query)
        except:
            job_result_rows = []
            logger.error('Failed to execute query due to %s', get_tracelog())
        
        query_result    = self.process_query_result(job_result_rows)
        
        logger.debug('query_result=%s', query_result)
        
        return query_result
        
    def prepare_query(self, **kwrgs):
        pass
    
    def process_query_result(self, job_result_rows):
        pass
    
class CategoryAndCountQueryBaseResource(QueryBaseResource):
    
    def process_query_result(self, job_result_rows):
        return process_job_result_into_category_and_count(job_result_rows)
    
class FieldnameAndValueQueryBaseResource(QueryBaseResource):
    
    def process_query_result(self, job_result_rows):
        return process_job_result_into_category_and_count(job_result_rows)    
    
    
        


'''
Created on 26 Jan 2021

@author: jacklok
'''
from trexanalytics.conf import UPSTREAM_UPDATED_DATETIME_FIELD_NAME, MERCHANT_DATASET, SYSTEM_DATASET
import uuid, logging
from trexmodel.models.datastore.analytic_models import UpstreamData
from trexanalytics.bigquery_table_template_config import REGISTERED_CUSTOMER_TEMPLATE, REGISTERED_MERCHANT_TEMPLATE, MERCHANT_REGISTERED_CUSTOMER_TEMPLATE,\
    CUSTOMER_TRANSACTION_TEMPLATE, MERCHANT_CUSTOMER_REWARD_TEMPLATE 
from trexlib.utils.google.bigquery_util import default_serializable
from datetime import datetime, timedelta
from trexmodel.models.datastore.transaction_models import CustomerTransactionWithRewardDetails
from trexanalytics import conf
from trexmodel.models.datastore.ndb_models import convert_to_serializable_value

__REGISTERED_MERCHANT_TEMPLATE_UPSTREAM_SCHEMA = { 
                                                'MerchantKey'           : 'key_in_str',
                                                'CompanyName'           : 'company_name',
                                                'RegisteredDateTime'    : 'registered_datetime',
                                            }

__REGISTERED_CUSTOMER_TEMPLATE_UPSTREAM_SCHEMA = {
                                                'UserKey'           : 'registered_user_acct_key',
                                                'CustomerKey'       : 'key_in_str',
                                                'MerchantKey'       : 'registered_merchant_acct_key',
                                                'DOB'               : 'birth_date',
                                                'Gender'            : 'gender',
                                                'MobilePhone'       : 'mobile_phone',
                                                'Email'             : 'email',
                                                'MobileAppInstall'  : 'mobile_app_installed',
                                                'RegisteredDateTime': 'registered_datetime',
                                                'RegisteredOutlet'  : 'registered_outlet_key',
                                                }

__MERCHANT_REGISTERED_CUSTOMER_TEMPLATE_UPSTREAM_SCHEMA = {
                                                        'UserKey'           : 'registered_user_acct_key',
                                                        'CustomerKey'       : 'key_in_str',
                                                        'DOB'               : 'birth_date',
                                                        'Gender'            : 'gender',
                                                        'MobilePhone'       : 'mobile_phone',
                                                        'Email'             : 'email',
                                                        'MobileAppInstall'  : 'mobile_app_installed',
                                                        'RegisteredDateTime': 'registered_datetime',
                                                        'RegisteredOutlet'  : 'registered_outlet_key',
                                                        }


__CUSTOMER_TRANSACTION_DATA_TEMPLATE_UPSTREAM_SCHEMA = {
                                            "UserKey"               : 'transact_user_acct_key',
                                            "CustomerKey"           : 'transact_customer_key',
                                            "MerchantKey"           : 'transact_merchant_acct_key',
                                            "TransactOutlet"        : 'transact_outlet_key',
                                            "TransactionId"         : 'transaction_id',
                                            "InvoiceId"             : 'invoice_id',
                                            "TransactAmount"        : 'transact_amount',
                                            "TransactDateTime"      : 'transact_datetime',
                                            }

__CUSTOMER_REWARD_DATA_TEMPLATE_UPSTREAM_SCHEMA = {
                                            "CustomerKey"           : 'transact_customer_key',
                                            "MerchantKey"           : 'transact_merchant_acct_key',
                                            "TransactOutlet"        : 'transact_outlet_key',
                                            "TransactionId"         : 'transaction_id',
                                            "TransactAmount"        : 'transact_amount',
                                            "TransactDateTime"      : 'transact_datetime',
                                            "RewardFormat"          : 'reward_format',
                                            "RewardAmount"          : 'reward_amount',
                                            "ExpiryDate"            : 'expiry_date',
                                            "RewardFormatKey"       : 'reward_format_key',
                                            "RewardedDateTime"      : 'rewarded_datetime',
                                            }


upstream_schema_config = {
                            REGISTERED_MERCHANT_TEMPLATE            : __REGISTERED_MERCHANT_TEMPLATE_UPSTREAM_SCHEMA,
                            REGISTERED_CUSTOMER_TEMPLATE            : __REGISTERED_CUSTOMER_TEMPLATE_UPSTREAM_SCHEMA,
                            MERCHANT_REGISTERED_CUSTOMER_TEMPLATE   : __MERCHANT_REGISTERED_CUSTOMER_TEMPLATE_UPSTREAM_SCHEMA,
                            CUSTOMER_TRANSACTION_TEMPLATE           : __CUSTOMER_TRANSACTION_DATA_TEMPLATE_UPSTREAM_SCHEMA,
                            MERCHANT_CUSTOMER_REWARD_TEMPLATE       : __CUSTOMER_REWARD_DATA_TEMPLATE_UPSTREAM_SCHEMA,
                            }

logger = logging.getLogger('upstream')

def __create_upstream(upstream_entity, upstream_template, dataset_name, table_name, update_datetime=None, **kwargs):
    upstream_json = {}
    if upstream_entity:
        schema = upstream_schema_config.get(upstream_template)
        for upstrem_field_name, attr_name in schema.items():
            upstream_json[upstrem_field_name] = default_serializable(getattr(upstream_entity, attr_name))
    
    if update_datetime is None:
        update_datetime = datetime.now() - timedelta(hours=int(conf.SERVER_DATETIME_GMT))
            
    upstream_json['Key'] = uuid.uuid1().hex
    upstream_json[UPSTREAM_UPDATED_DATETIME_FIELD_NAME] = default_serializable(update_datetime)
    
    for key, value in kwargs.items():
        upstream_json[key] = convert_to_serializable_value(value)
    
    logger.debug('-------------------------------------------------')
    logger.debug('upstream_json=%s', upstream_json)
    logger.debug('-------------------------------------------------')
    UpstreamData.create(dataset_name, table_name, upstream_template, [upstream_json])
    

def create_registered_customer_upstream_for_system(customer):
    update_datetime     = datetime.now()
    table_name          = REGISTERED_CUSTOMER_TEMPLATE
    final_table_name    = '{}_{}'.format(table_name, update_datetime.strftime('%Y%m%d'))
    
    __create_upstream(customer, REGISTERED_CUSTOMER_TEMPLATE, SYSTEM_DATASET, final_table_name, update_datetime=update_datetime)
    
def create_merchant_registered_customer_upstream_for_merchant(customer):
    update_datetime     = datetime.now()
    table_name          = MERCHANT_REGISTERED_CUSTOMER_TEMPLATE
    merchant_acct       = customer.registered_merchant_acct
    account_code        = merchant_acct.account_code.replace('-','')
    final_table_name    = '{}_{}_{}'.format(table_name, account_code, update_datetime.strftime('%Y%m%d'))
    
    __create_upstream(customer, MERCHANT_REGISTERED_CUSTOMER_TEMPLATE, MERCHANT_DATASET, final_table_name, update_datetime=update_datetime)    
    
def create_merchant_customer_transaction_upstream_for_merchant(transaction_details, update_datetime=None):
    if update_datetime is None:
        update_datetime     = datetime.now()
        
    table_name          = CUSTOMER_TRANSACTION_TEMPLATE
    merchant_acct       = transaction_details.transact_merchant_acct
    account_code        = merchant_acct.account_code.replace('-','')
    final_table_name    = '{}_{}_{}'.format(table_name, account_code, update_datetime.strftime('%Y%m%d'))
    
    __create_upstream(transaction_details, CUSTOMER_TRANSACTION_TEMPLATE, MERCHANT_DATASET, final_table_name, update_datetime=update_datetime, Reverted=False, RevertedDateTime=None)

def create_merchant_customer_reward_upstream_for_merchant(transaction_details, reward_details, update_datetime=None):
    if update_datetime is None:
        update_datetime     = datetime.now() - timedelta(hours=int(conf.SERVER_DATETIME_GMT))
    
    table_name          = MERCHANT_CUSTOMER_REWARD_TEMPLATE
    merchant_acct       = transaction_details.transact_merchant_acct
    account_code        = merchant_acct.account_code.replace('-','')
    final_table_name    = '{}_{}_{}'.format(table_name, account_code, update_datetime.strftime('%Y%m%d'))
    
    transaction_details_with_reward_details = CustomerTransactionWithRewardDetails(transaction_details, reward_details)
    
    __create_upstream(transaction_details_with_reward_details, MERCHANT_CUSTOMER_REWARD_TEMPLATE, MERCHANT_DATASET, final_table_name, update_datetime=update_datetime)






'''
Created on 11 Jan 2021

@author: jacklok
'''
from google.cloud import bigquery


__REGISTERED_USER_SCHEMA = {
                        
                            #customer key
                            bigquery.SchemaField('Key', 'STRING'),
                            bigquery.SchemaField('UserKey', 'STRING'),
                            bigquery.SchemaField('DOB', 'DATE'),
                            bigquery.SchemaField('Gender', 'STRING'),
                            bigquery.SchemaField('MobilePhone', 'STRING'),
                            bigquery.SchemaField('Email', 'STRING'),
                            bigquery.SchemaField('MobileAppDownload', 'BOOLEAN'),
                            bigquery.SchemaField('RegisteredDateTime', 'DATETIME'),
                            bigquery.SchemaField('UpdatedDateTime', 'DATETIME'),
                            }

__REGISTERED_CUSTOMER_SCHEMA = {
                        
                            #customer key
                            bigquery.SchemaField('Key', 'STRING'),
                            bigquery.SchemaField('UserKey', 'STRING'),
                            bigquery.SchemaField('CustomerKey', 'STRING'),
                            bigquery.SchemaField('MerchantKey', 'STRING'),
                            bigquery.SchemaField('DOB', 'DATE'),
                            bigquery.SchemaField('Gender', 'STRING'),
                            bigquery.SchemaField('MobilePhone', 'STRING'),
                            bigquery.SchemaField('Email', 'STRING'),
                            bigquery.SchemaField('MobileAppInstall', 'BOOLEAN'),
                            bigquery.SchemaField('RegisteredDateTime', 'DATETIME'),
                            bigquery.SchemaField('RegisteredOutlet', 'STRING'),
                            bigquery.SchemaField('UpdatedDateTime', 'DATETIME'),
                            }

__MERCHANT_REGISTERED_CUSTOMER_SCHEMA = {
                        
                            #customer key
                            bigquery.SchemaField('Key', 'STRING'),
                            bigquery.SchemaField('UserKey', 'STRING'),
                            bigquery.SchemaField('CustomerKey', 'STRING'),
                            bigquery.SchemaField('DOB', 'DATE'),
                            bigquery.SchemaField('Gender', 'STRING'),
                            bigquery.SchemaField('MobilePhone', 'STRING'),
                            bigquery.SchemaField('Email', 'STRING'),
                            bigquery.SchemaField('MobileAppInstall', 'BOOLEAN'),
                            bigquery.SchemaField('RegisteredDateTime', 'DATETIME'),
                            bigquery.SchemaField('RegisteredOutlet', 'STRING'),
                            bigquery.SchemaField('UpdatedDateTime', 'DATETIME'),
                            }

__MERCHANT_CUSTOMER_TRANSACTION_SCHEMA = {
                        
                            #customer key
                            bigquery.SchemaField('Key', 'STRING'),
                            bigquery.SchemaField('UserKey', 'STRING'),
                            bigquery.SchemaField('CustomerKey', 'STRING'),
                            bigquery.SchemaField('MerchantKey', 'STRING'),
                            bigquery.SchemaField('TransactOutlet', 'STRING'),
                            bigquery.SchemaField('TransactionId', 'STRING'),
                            bigquery.SchemaField('InvoiceId', 'STRING'),
                            bigquery.SchemaField('TransactAmount', 'FLOAT64'),
                            bigquery.SchemaField('TransactDateTime', 'DATETIME'),
                            bigquery.SchemaField('UpdatedDateTime', 'DATETIME'),
                            }

__MERCHANT_CUSTOMER_REWARD_SCHEMA = {
                        
                            #customer key
                            bigquery.SchemaField('Key', 'STRING'),
                            bigquery.SchemaField('CustomerKey', 'STRING'),
                            bigquery.SchemaField('MerchantKey', 'STRING'),
                            bigquery.SchemaField('TransactOutlet', 'STRING'),
                            bigquery.SchemaField('TransactionId', 'STRING'),
                            bigquery.SchemaField('TransactAmount', 'FLOAT64'),
                            bigquery.SchemaField('TransactDateTime', 'DATETIME'),
                            bigquery.SchemaField('RewardFormat', 'STRING'),
                            bigquery.SchemaField('RewardAmount', 'FLOAT64'),
                            bigquery.SchemaField('ExpiryDate', 'DATE'),
                            bigquery.SchemaField('RewardFormatKey', 'STRING'),
                            bigquery.SchemaField('RewardedDateTime', 'DATETIME'),
                            bigquery.SchemaField('Reverted', 'BOOLEAN'),
                            bigquery.SchemaField('RevertedDateTime', 'DATETIME'),
                            bigquery.SchemaField('UpdatedDateTime', 'DATETIME'),
                            }

__REGISTERED_MERCHANT_SCHEMA = {
                        
                            #customer key
                            bigquery.SchemaField('Key', 'STRING'),
                            bigquery.SchemaField('MerchantKey', 'STRING'),
                            bigquery.SchemaField('CompanyName', 'STRING'),
                            bigquery.SchemaField('RegisteredDateTime', 'DATETIME'),
                            bigquery.SchemaField('UpdatedDateTime', 'DATETIME'),
                            }
    

REGISTERED_USER_TEMPLATE                    = 'registered_user'
REGISTERED_CUSTOMER_TEMPLATE                = 'registered_customer'
CUSTOMER_TRANSACTION_TEMPLATE               = 'customer_transaction'
REGISTERED_MERCHANT_TEMPLATE                = 'registered_merchant'
MERCHANT_REGISTERED_CUSTOMER_TEMPLATE       = 'merchant_registered_customer'
MERCHANT_CUSTOMER_REWARD_TEMPLATE           = 'customer_reward'
    

TABLE_SCHEME_TEMPLATE = {
                    REGISTERED_USER_TEMPLATE                : __REGISTERED_USER_SCHEMA,
                    REGISTERED_CUSTOMER_TEMPLATE            : __REGISTERED_CUSTOMER_SCHEMA,
                    REGISTERED_MERCHANT_TEMPLATE            : __REGISTERED_MERCHANT_SCHEMA,
                    MERCHANT_REGISTERED_CUSTOMER_TEMPLATE   : __MERCHANT_REGISTERED_CUSTOMER_SCHEMA,
                    CUSTOMER_TRANSACTION_TEMPLATE           : __MERCHANT_CUSTOMER_TRANSACTION_SCHEMA,
                    MERCHANT_CUSTOMER_REWARD_TEMPLATE       : __MERCHANT_CUSTOMER_REWARD_SCHEMA
    }



'''
Created on 11 Jan 2021

@author: jacklok
'''

from flask import Blueprint, render_template, session, abort, redirect, url_for
import logging, json
from flask.globals import current_app
from flask.json import jsonify
from json import JSONEncoder
from datetime import datetime, date
from trexlib.utils.google.bigquery_util import create_table_from_template
from trexanalytics import conf as analytics_conf

table_template_bp = Blueprint('table_template_bp', __name__,
                     template_folder='templates',
                     static_folder='static',
                     url_prefix='/template')

logger = logging.getLogger('debug')

@table_template_bp.route('/create/dataset/<dataset_name>/table-template/<table_name>')
def create_from_table_template(dataset_name, table_name): 
    created_table = create_table_from_template(dataset_name, table_name)
    
    return str(created_table), 200



'''
Created on 3 May 2021

@author: jacklok
'''

from flask import Blueprint
import logging
from trexanalytics.controllers.bigquery.query_base_routes import CategoryAndCountQueryBaseResource,\
    QueryBaseResource
from flask_restful import Api
from trexanalytics.conf import BIGQUERY_GCLOUD_PROJECT_ID, MERCHANT_DATASET


transaction_analytics_data_bp = Blueprint('transaction_analytics_data_bp', __name__,
                                 template_folder='templates',
                                 static_folder='static',
                                 url_prefix='/analytics/transaction')

logger = logging.getLogger('query')

query_transaction_data_api = Api(transaction_analytics_data_bp)

@transaction_analytics_data_bp.route('/index')
def query_transaction_index(): 
    return 'ping', 200


def process_transaction_result_into_fieldname_and_value(job_result_rows):
    row_list = []
    for row in job_result_rows:
        #logger.debug(row)
        column_dict = {}
        column_dict['date_range']           = row.date_range
        column_dict['sumTransactAmount']    = row.sumTransactAmount
        column_dict['transactionCount']     = row.transactionCount
        
        row_list.append(column_dict)
    
    return row_list

def process_transaction_result_without_dange_range_into_fieldname_and_value(job_result_rows):
    row_list = []
    for row in job_result_rows:
        #logger.debug(row)
        column_dict = {}
        column_dict['totalTransactAmount']      = row.totalTransactAmount
        column_dict['transactionCount']         = row.transactionCount
        
        row_list.append(column_dict)
    
    return row_list     

class CustomerTransactionQueryBase(QueryBaseResource):
    def process_query_result(self, job_result_rows):
        return process_transaction_result_into_fieldname_and_value(job_result_rows)
    

class QueryAllCustomerTransactionByYearMonth(CustomerTransactionQueryBase):
    def prepare_query(self, **kwrgs):
        
        year_month_from   = kwrgs.get('year_month_from')
        year_month_to     = kwrgs.get('year_month_to')
        
        where_condition  = ''
        
        if year_month_from and year_month_to:
            where_condition = "WHERE _TABLE_SUFFIX BETWEEN '{year_month_from}' and '{year_month_to}'".format(year_month_from=year_month_from, 
                                                                                                         year_month_to=year_month_to)
        
        query = '''
            SELECT FORMAT_DATETIME('%Y-%m', TransactDateTime) as date_range, sum(TotalTransactAmount) as sumTransactAmount, count(*) as transactionCount
            FROM (
                        SELECT TransactDateTime, SUM(TransactAmount) as TotalTransactAmount
                        FROM (
                            SELECT TransactDateTime, TransactionId, TransactAmount
                            FROM `{project_id}.{dataset_name}.customer_transaction_*`
                            {where_condition}
                            GROUP BY TransactDateTime, TransactionId, TransactAmount
                        )
                        GROUP BY TransactDateTime
                     
            ) GROUP BY date_range  
            order by date_range          
            '''.format(project_id=BIGQUERY_GCLOUD_PROJECT_ID, dataset_name=MERCHANT_DATASET, where_condition=where_condition)    
            
        logger.debug('QueryAllCustomerTransactionByYearMonth: query=%s', query)
    
        return query 
    
class QueryMerchantCustomerTransactionByYearMonth(CustomerTransactionQueryBase):
    def prepare_query(self, **kwrgs):
        account_code      = kwrgs.get('account_code')
        year_month_from   = kwrgs.get('year_month_from')
        year_month_to     = kwrgs.get('year_month_to')
        
        account_code = account_code.replace('-','')
        
        where_condition  = ''
        
        if year_month_from and year_month_to:
            where_condition = "WHERE _TABLE_SUFFIX BETWEEN '{year_month_from}' and '{year_month_to}'".format(year_month_from=year_month_from, 
                                                                                                         year_month_to=year_month_to)
        
        query = '''
            SELECT FORMAT_DATETIME('%Y-%m', TransactDateTime) as date_range, sum(TotalTransactAmount) as sumTransactAmount, count(*) as transactionCount
            FROM (
                        SELECT TransactDateTime, SUM(TransactAmount) as TotalTransactAmount
                        FROM (
                            SELECT TransactDateTime, TransactionId, TransactAmount
                            FROM `{project_id}.{dataset_name}.customer_transaction_{account_code}_*`
                            {where_condition}
                            GROUP BY TransactDateTime, TransactionId, TransactAmount
                        )
                        GROUP BY TransactDateTime
                     
            ) GROUP BY date_range  
            order by date_range          
            '''.format(project_id=BIGQUERY_GCLOUD_PROJECT_ID, dataset_name=MERCHANT_DATASET, where_condition=where_condition, account_code=account_code)    
            
        logger.debug('QueryMerchantCustomerTransactionByYearMonth: query=%s', query)
    
        return query
    
class QueryMerchantCustomerTransactionByDateRange(CustomerTransactionQueryBase):
    def prepare_query(self, **kwrgs):
        account_code      = kwrgs.get('account_code')
        date_range_from   = kwrgs.get('date_range_from')
        date_range_to     = kwrgs.get('date_range_to')
        
        account_code = account_code.replace('-','')
        
        where_condition  = ''
        
        if date_range_from and date_range_to:
            where_condition = "WHERE _TABLE_SUFFIX BETWEEN '{date_range_from}' and '{date_range_to}'".format(date_range_from=date_range_from, 
                                                                                                         date_range_to=date_range_to)
        
        query = '''
            SELECT FORMAT_DATETIME('%Y-%m-%d', TransactDateTime) as date_range, sum(TotalTransactAmount) as sumTransactAmount, count(*) as transactionCount
            FROM (
                        SELECT TransactDateTime, SUM(TransactAmount) as TotalTransactAmount
                        FROM (
                            SELECT TransactDateTime, TransactionId, TransactAmount
                            FROM `{project_id}.{dataset_name}.customer_transaction_{account_code}_*`
                            {where_condition}
                            GROUP BY TransactDateTime, TransactionId, TransactAmount
                        )
                        GROUP BY TransactDateTime
                     
            ) GROUP BY date_range    
            order by date_range        
            '''.format(project_id=BIGQUERY_GCLOUD_PROJECT_ID, dataset_name=MERCHANT_DATASET, where_condition=where_condition, account_code=account_code)    
            
        logger.debug('QueryMerchantCustomerTransactionByYearMonth: query=%s', query)
    
        return query
    
class QueryMerchantCustomerTransactionTotalByDateRange(CustomerTransactionQueryBase):
    def prepare_query(self, **kwrgs):
        account_code      = kwrgs.get('account_code')
        date_range_from   = kwrgs.get('date_range_from')
        date_range_to     = kwrgs.get('date_range_to')
        
        account_code = account_code.replace('-','')
        
        where_condition  = ''
        
        if date_range_from and date_range_to:
            where_condition = "WHERE _TABLE_SUFFIX BETWEEN '{date_range_from}' and '{date_range_to}'".format(date_range_from=date_range_from, 
                                                                                                         date_range_to=date_range_to)
        
        query = '''
            
                        SELECT SUM(TransactAmount) as totalTransactAmount, count(*) as transactionCount
                        FROM (
                            SELECT TransactDateTime, TransactionId, TransactAmount
                            FROM `{project_id}.{dataset_name}.customer_transaction_{account_code}_*`
                            {where_condition}
                            GROUP BY TransactDateTime, TransactionId, TransactAmount
                        )
                        
                     
                   
            '''.format(project_id=BIGQUERY_GCLOUD_PROJECT_ID, dataset_name=MERCHANT_DATASET, where_condition=where_condition, account_code=account_code)    
            
        logger.debug('QueryMerchantCustomerTransactionByYearMonth: query=%s', query)
    
        return query    
    
    def process_query_result(self, job_result_rows):
        return process_transaction_result_without_dange_range_into_fieldname_and_value(job_result_rows)
    

query_transaction_data_api.add_resource(QueryAllCustomerTransactionByYearMonth,   '/all-transaction-by-year-month')
query_transaction_data_api.add_resource(QueryMerchantCustomerTransactionByYearMonth,   '/merchant-transaction-by-year-month')
query_transaction_data_api.add_resource(QueryMerchantCustomerTransactionByDateRange,   '/merchant-transaction-by-date-range')
query_transaction_data_api.add_resource(QueryMerchantCustomerTransactionTotalByDateRange,   '/merchant-transaction-total-by-date-range')





'''
Created on 14 Jan 2021

@author: jacklok
'''

import unittest, logging, sys
from trexanalytics.controllers.importdata.import_customer_data_routes import import_all_registered_customer_function
from trexmodel.utils.model.model_util import read_service_account_file
from trexanalytics import conf as analytics_conf
from trexmodel import conf as model_conf
from datetime import datetime


#dataset_name = 'system_analytics'
dataset_name = 'unittest'

class TestImportCustomerData(unittest.TestCase):
    
    def setUp(self):
        credential_filepath                         = model_conf.DATASTORE_SERVICE_ACCOUNT_KEY_FILEPATH
        (credential_config, data)                   = read_service_account_file(credential_filepath=credential_filepath)
        self.credential_config                      = credential_config
        
    def tearDown(self):
        pass

    def test_import_registered_customer_data(self):
        
        import_count = import_all_registered_customer_function(credential_info=self.credential_config, dataset_name=dataset_name, 
                                                               partition_month=False, limit=10)
        

logging.basicConfig(stream=sys.stderr, level=logging.DEBUG)

if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()


'''
Created on 19 Jan 2021

@author: jacklok
'''
import unittest, logging, sys
from main import app

logger = logging.getLogger('unittest')

system_dataset_name = 'system'

class QueryCustomerUnitTestCase(unittest.TestCase):
    def test_query_registered_customer_blueprint_index(self):
        response = app.test_client().get('/biqquery/customer/index')

        assert response.status_code == 200
        assert response.data == b'ping'
       
    def test_query_registered_customer_by_year_month(self):
        response = app.test_client().get('/biqquery/customer/registered-customer-by-year-month', json={'year_month_from': 202101, 'year_month_to': 202112})
        
        logger.debug('response=%s', response)

        assert response.status_code == 200
        #assert response.data == b'ping'    
        
        

if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()


'''
Created on 15 Jan 2021

@author: jacklok
'''

import unittest, logging, sys
from trexanalytics.controllers.importdata.import_customer_data_routes import import_merchant_registered_customer_function, import_merchant_customer_changes
from trexmodel.models.datastore.customer_models import Customer
from trexmodel.utils.model.model_util import read_service_account_file
from trexanalytics import conf as analytics_conf
from trexmodel import conf as model_conf
from datetime import datetime, date
from trexmodel.utils.model.model_util import create_db_client 

dataset_name = 'unittest'

class TestImportMerchantCustomerData(unittest.TestCase):
    
    def setUp(self):
        credential_filepath                         = model_conf.DATASTORE_SERVICE_ACCOUNT_KEY_FILEPATH
        (credential_config, data)                   = read_service_account_file(credential_filepath=credential_filepath)
        self.credential_config                      = credential_config
        
    def tearDown(self):
        pass
    
    
    def test_import_registered_customer_data(self):
        merchant_acct_key = 'ahBufnRyZXgtYWRtaW4tZGV2chkLEgxNZXJjaGFudEFjY3QYgICAmMX2ggoM'
        
        import_merchant_registered_customer_function(credential_info=self.credential_config, dataset_name=dataset_name,
                                                                    merchant_acct_key = merchant_acct_key, 
                                                                    partition_date=True, limit=10)
        
    def test_import_changed_customer_data(self):
        db_client = create_db_client(info=self.credential_config, caller_info="count_import_all_registered_customer_function")
        with db_client.context():
            customers = Customer.list_all(offset=0, limit=1)
        
        assert len(customers)>0
        customer = customers[0]
        
        customer.birth_date = date(1977,2,2)
        
        assert customer is not None
        errors = import_merchant_customer_changes(customer, credential_info=self.credential_config, dataset_name=dataset_name,
                                                                    )    
        assert len(errors)==0

logging.basicConfig(stream=sys.stderr, level=logging.DEBUG)

if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()


'''
Created on 11 Jan 2021

@author: jacklok
'''

import unittest, logging, sys
from trexlib.utils.google.bigquery_util import create_bigquery_client, create_table_from_template, list_all_table_full_id_from_dataset, detele_table_from_dataset, is_dataset_exist, detele_dataset
from trexanalytics.bigquery_table_template_config import REGISTERED_CUSTOMER_TEMPLATE, REGISTERED_MERCHANT_TEMPLATE, REGISTERED_USER_TEMPLATE, TABLE_SCHEME_TEMPLATE
from trexanalytics.conf import BIGQUERY_SERVICE_CREDENTIAL_PATH
logger = logging.getLogger('unittest')

dataset_name    = 'unittest'

class CreateBigqueryTableUnitTestCase(unittest.TestCase):
    def setUp(self):
        self.bq_client = create_bigquery_client(credential_filepath=BIGQUERY_SERVICE_CREDENTIAL_PATH)

        
    def tearDown(self):
        pass
    
    
    def test_create_registred_customer_table(self):
        table_name      = REGISTERED_CUSTOMER_TEMPLATE
        
        created_table = create_table_from_template(dataset_name, table_name, TABLE_SCHEME_TEMPLATE.get(REGISTERED_CUSTOMER_TEMPLATE), bigquery_client=self.bq_client)
        
        assert created_table is not None
        
    def test_create_registred_merchant_table(self):
        table_name      = REGISTERED_MERCHANT_TEMPLATE
        
        created_table = create_table_from_template(dataset_name, table_name, TABLE_SCHEME_TEMPLATE.get(REGISTERED_MERCHANT_TEMPLATE), bigquery_client=self.bq_client)
        
        assert created_table is not None
        
    def test_create_registred_user_table(self):
        table_name      = REGISTERED_USER_TEMPLATE
        
        created_table = create_table_from_template(dataset_name, table_name, TABLE_SCHEME_TEMPLATE.get(REGISTERED_USER_TEMPLATE), bigquery_client=self.bq_client)
        
        assert created_table is not None   
        
    
    def test_list_all_table(self):
        tables_list = list_all_table_full_id_from_dataset(dataset_name, bigquery_client=self.bq_client)
        
        assert tables_list is not None           
        assert len(tables_list) == 3
        
        logger.debug('tables_list=%s', tables_list)
        
        #test delete all tables
        
    '''
    def test_delete_registred_customer_table(self):
        dataset_name    = 'unitest'
        table_name      = REGISTERED_CUSTOMER_TEMPLATE
        
        deleted_table = detele_table_from_dataset(dataset_name, table_name)
        
        assert deleted_table is not None    
    '''
        
    
    '''
    
    def test_check_dataset_is_exist(self):
        #bq_client = create_bigquery_client(credential_filepath=BIGQUERY_SERVICE_CREDENTIAL_PATH)
        is_exist = is_dataset_exist(dataset_name, bigquery_client=self.bq_client)
        
        assert is_exist is True
    '''
        
    '''
    def test_delete_dataset(self):
        dataset_name    = 'unitest'
        deleted_dataset = detele_dataset(dataset_name)
        
        assert deleted_dataset is not None    
    '''    
        
logging.basicConfig(stream=sys.stderr, level=logging.DEBUG)        



'''
Created on 19 Jan 2021

@author: jacklok
'''
import unittest
from main import app


class Test(unittest.TestCase):


    def test_ping(self):
        response = app.test_client().get('/ping')

        assert response.status_code == 200
        assert response.data == b'ping'



if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.test_hello']
    unittest.main()


'''
Created on 18 Jan 2021

@author: jacklok
'''
from flask import Blueprint
import logging
from trexlib.utils.google.bigquery_util import process_job_result_into_category_and_count, execute_query
from trexanalytics.controllers.bigquery.query_base_routes import CategoryAndCountQueryBaseResource,\
    QueryBaseResource
from flask_restful import Api
from trexanalytics.conf import BIGQUERY_GCLOUD_PROJECT_ID, SYSTEM_DATASET, MERCHANT_DATASET


customer_analytics_data_bp = Blueprint('customer_analytics_data_bp', __name__,
                                 template_folder='templates',
                                 static_folder='static',
                                 url_prefix='/analytics/customer')

logger = logging.getLogger('query')

query_customer_data_api = Api(customer_analytics_data_bp)

@customer_analytics_data_bp.route('/index')
def query_customer_index(): 
    return 'ping', 200


class QueryAllCustomerGrowthByYearMonth(CategoryAndCountQueryBaseResource):
    def prepare_query(self, **kwrgs):
        
        year_month_from   = kwrgs.get('year_month_from')
        year_month_to     = kwrgs.get('year_month_to')
        
        where_condition  = ''
        
        if year_month_from and year_month_to:
            where_condition = "WHERE _TABLE_SUFFIX BETWEEN '{year_month_from}' and '{year_month_to}'".format(year_month_from=year_month_from, 
                                                                                                         year_month_to=year_month_to)
        
        query = '''
            SELECT FORMAT_DATETIME('%Y-%m', RegisteredDateTime) as year_month, sum(count_by_date) as count
            FROM (
            SELECT RegisteredDateTime, count(*) as count_by_date FROM
                (
                        SELECT CustomerKey, RegisteredDateTime
                        FROM `{project_id}.{dataset_name}.registered_customer_*`
                        {where_condition}
                        GROUP BY CustomerKey, RegisteredDateTime
                ) GROUP BY RegisteredDateTime       
            ) GROUP BY year_month            
            '''.format(project_id=BIGQUERY_GCLOUD_PROJECT_ID, dataset_name=SYSTEM_DATASET, where_condition=where_condition)    
            
        logger.debug('execute_all_registered_customer_by_year_month: query=%s', query)
    
        return query 
    
class QueryMerchantCustomerGrowthByYearMonth(CategoryAndCountQueryBaseResource):
    def prepare_query(self, **kwrgs):
        account_code      = kwrgs.get('account_code')
        year_month_from   = kwrgs.get('year_month_from')
        year_month_to     = kwrgs.get('year_month_to')
        
        account_code = account_code.replace('-','')
        
        where_condition  = ''
        
        if year_month_from and year_month_to:
            where_condition = "WHERE _TABLE_SUFFIX BETWEEN '{year_month_from}' and '{year_month_to}'".format(year_month_from=year_month_from, 
                                                                                                         year_month_to=year_month_to)
        
        query = '''
            SELECT FORMAT_DATETIME('%Y-%m', RegisteredDateTime) as year_month, sum(count_by_date) as count
            FROM (
            SELECT RegisteredDateTime, count(*) as count_by_date FROM
                (
                        SELECT CustomerKey, RegisteredDateTime
                        FROM `{project_id}.{dataset_name}.merchant_registered_customer_{account_code}_*`
                        {where_condition}
                        GROUP BY CustomerKey, RegisteredDateTime
                ) GROUP BY RegisteredDateTime       
            ) GROUP BY year_month            
            '''.format(project_id=BIGQUERY_GCLOUD_PROJECT_ID, dataset_name=MERCHANT_DATASET, where_condition=where_condition, account_code=account_code)    
            
        logger.debug('execute_all_registered_customer_by_year_month: query=%s', query)
    
        return query 
    
class QueryMerchantCustomerCountByDateRange(QueryBaseResource):
    def prepare_query(self, **kwrgs):
        account_code      = kwrgs.get('account_code')
        date_range_from   = kwrgs.get('date_range_from')
        date_range_to     = kwrgs.get('date_range_to')
        
        account_code = account_code.replace('-','')
        
        where_condition  = ''
        
        if date_range_from and date_range_to:
            where_condition = "WHERE _TABLE_SUFFIX BETWEEN '{date_range_from}' and '{date_range_to}'".format(date_range_from=date_range_from, 
                                                                                                         date_range_to=date_range_to)
        
        query = '''
            SELECT count(*) as customerCount FROM
                (
                        SELECT CustomerKey, RegisteredDateTime
                        FROM `{project_id}.{dataset_name}.merchant_registered_customer_{account_code}_*`
                        {where_condition}
                        GROUP BY CustomerKey, RegisteredDateTime
                )       
                        
            '''.format(project_id=BIGQUERY_GCLOUD_PROJECT_ID, dataset_name=MERCHANT_DATASET, where_condition=where_condition, account_code=account_code)    
            
        logger.debug('QueryMerchantCustomerCountByDateRange: query=%s', query)
    
        return query 
    
    def process_query_result(self, job_result_rows):
        row_list = []
        for row in job_result_rows:
            #logger.debug(row)
            column_dict = {}
            column_dict['customerCount']        = row.customerCount
            
            row_list.append(column_dict)
        
        return row_list 
    
class QueryMerchantCustomerGenderByDateRange(QueryBaseResource):
    def prepare_query(self, **kwrgs):
        account_code      = kwrgs.get('account_code')
        date_range_from   = kwrgs.get('date_range_from')
        date_range_to     = kwrgs.get('date_range_to')
        
        account_code = account_code.replace('-','')
        
        where_condition  = ''
        
        if date_range_from and date_range_to:
            where_condition = "WHERE _TABLE_SUFFIX BETWEEN '{date_range_from}' and '{date_range_to}'".format(date_range_from=date_range_from, 
                                                                                                         date_range_to=date_range_to)
        
        query = '''
            SELECT coalesce(Gender,'u') as gender, count(*) as genderCount FROM
                (
                
                SELECT
                  checking.Key, checking.CustomerKey, checking.UpdatedDateTime, checking.Gender as gender
                FROM
                  (
                  SELECT
                     CustomerKey, MAX(UpdatedDateTime) AS latestUpdatedDateTime
                   FROM
                    `{project_id}.{dataset_name}.merchant_registered_customer_{account_code}_*`
                    {where_condition}
                    
                  GROUP BY
                 CustomerKey
                 ) 
                 AS latest_customers
            INNER JOIN 
            (
              SELECT 
              Key, CustomerKey, UpdatedDateTime, Gender
              FROM
              `{project_id}.{dataset_name}.merchant_registered_customer_{account_code}_*`
              {where_condition} 
                 
            ) as checking
            ON
              checking.CustomerKey = latest_customers.CustomerKey
              AND
              checking.UpdatedDateTime=latest_customers.latestUpdatedDateTime
            ) group by gender      
                        
            '''.format(project_id=BIGQUERY_GCLOUD_PROJECT_ID, dataset_name=MERCHANT_DATASET, where_condition=where_condition, account_code=account_code)    
            
        logger.debug('QueryMerchantCustomerCountByDateRange: query=%s', query)
    
        return query 
    
    def process_query_result(self, job_result_rows):
        row_list = []
        for row in job_result_rows:
            #logger.debug(row)
            column_dict = {}
            column_dict['gender']        = row.gender
            column_dict['count']        = row.genderCount
            
            row_list.append(column_dict)
        
        return row_list 
    
class QueryMerchantCustomerAgeGroupByDateRange(QueryBaseResource):
    def prepare_query(self, **kwrgs):
        account_code      = kwrgs.get('account_code')
        date_range_from   = kwrgs.get('date_range_from')
        date_range_to     = kwrgs.get('date_range_to')
        
        account_code = account_code.replace('-','')
        
        where_condition  = ''
        
        if date_range_from and date_range_to:
            where_condition = "WHERE _TABLE_SUFFIX BETWEEN '{date_range_from}' and '{date_range_to}'".format(date_range_from=date_range_from, 
                                                                                                         date_range_to=date_range_to)
        
        query = '''
            SELECT age_group, sum(group_count) as count  FROM (

                SELECT age_group, count(*) as group_count FROM (
                    SELECT CASE 
                       WHEN age >=  0 AND age < 20 THEN '0-19'
                       WHEN age >= 20 AND age < 30 THEN '20-29'
                       WHEN age >= 30 AND age < 40 THEN '30-39'
                       WHEN age >= 40 AND age < 50 THEN '40-49'
                       WHEN age >= 50 AND age < 60 THEN '50-59'
                       WHEN age >= 60 AND age < 70 THEN '60-69'
                       WHEN age >= 70 AND age < 80 THEN '70-79'
                       WHEN age >= 80              THEN '>=80'
                       ELSE 'unknown' END as age_group
            
                        FROM(
            
                            SELECT IFNULL(DATE_DIFF(CURRENT_DATE(),DOB, YEAR), -1) as age
                                FROM(
            
                                SELECT
                                  checking.Key, checking.CustomerKey, checking.UpdatedDateTime, checking.DOB as DOB
                                FROM
                                  (
                                  SELECT
                                     CustomerKey, MAX(UpdatedDateTime) AS latestUpdatedDateTime
                                   FROM
                                     `{project_id}.{dataset_name}.merchant_registered_customer_{account_code}_*`
                                     {where_condition} 
            
                                   GROUP BY
                                     CustomerKey
                                     ) 
                                     AS latest_customers
                                INNER JOIN
                                (
                                  SELECT 
                                  Key, CustomerKey, UpdatedDateTime, DOB
                                  FROM
                                  `{project_id}.{dataset_name}.merchant_registered_customer_{account_code}_*`
                                  {where_condition} 
            
                                ) as checking
                                
                            ON
                                
                            checking.CustomerKey = latest_customers.CustomerKey
                            AND
                            checking.UpdatedDateTime=latest_customers.latestUpdatedDateTime
                        ) 
                    )                 
                ) group by age_group
            ) group by age_group
                        
            '''.format(project_id=BIGQUERY_GCLOUD_PROJECT_ID, dataset_name=MERCHANT_DATASET, where_condition=where_condition, account_code=account_code)    
            
        logger.debug('QueryMerchantCustomerCountByDateRange: query=%s', query)
    
        return query 
    
    def process_query_result(self, job_result_rows):
        row_list = []
        for row in job_result_rows:
            #logger.debug(row)
            column_dict = {}
            column_dict['age_group']    = row.age_group
            column_dict['count']        = row.count
            
            row_list.append(column_dict)
        
        return row_list                   
    

def execute_all_registered_customer_by_year_month(bg_client, project_id, dataset_name):
    
    query = '''
            SELECT FORMAT_DATETIME('%Y-%m-%d', RegisteredDateTime) as year_month, sum(count_by_date) as count
            FROM (
            SELECT RegisteredDateTime, count(*) as count_by_date
                        FROM `{project_id}.{dataset_name}.registered_customer_*`
                        GROUP BY RegisteredDateTime
            ) GROUP BY year_month            
            '''.format(project_id=project_id, dataset_name=dataset_name)
    
    logger.debug('execute_all_registered_customer_by_year_month: query=%s', query)
    
    return execute_query(bg_client, query)


def process_all_registered_customer_by_year_month(job_result_rows):
    return process_job_result_into_category_and_count(job_result_rows)

query_customer_data_api.add_resource(QueryAllCustomerGrowthByYearMonth,   '/all-customer-growth-by-year-month')
query_customer_data_api.add_resource(QueryMerchantCustomerGrowthByYearMonth,   '/merchant-customer-growth-by-year-month')
query_customer_data_api.add_resource(QueryMerchantCustomerCountByDateRange,   '/merchant-customer-count-by-date-range')
query_customer_data_api.add_resource(QueryMerchantCustomerGenderByDateRange,   '/merchant-customer-gender-by-date-range')
query_customer_data_api.add_resource(QueryMerchantCustomerAgeGroupByDateRange,   '/merchant-customer-age-group-by-date-range')



'''
Created on 12 Jan 2021

@author: jacklok
'''

from flask import Blueprint, render_template, session, abort, redirect, url_for, current_app
import logging, json, uuid
from flask.globals import current_app
from trexlib.utils.google.bigquery_util import create_table_from_template, create_bigquery_client, stream_data_by_datetime_partition
from trexanalytics import conf as analytics_conf
from trexanalytics.bigquery_table_template_config import REGISTERED_MERCHANT_TEMPLATE
from trexmodel.models.datastore.merchant_models import MerchantAcct

from trexmodel.utils.model.model_util import create_db_client 
from trexmodel.conf import MAX_FETCH_RECORD
from datetime import datetime
from trexlib.utils.log_util import get_tracelog
from trexanalytics.controllers.importdata.import_data_base_routes import TriggerImportDataBaseResource, InitImportDataBaseResource, ImportDataBaseResource
from flask_restful import Api
from trexanalytics.bigquery_table_template_config import TABLE_SCHEME_TEMPLATE


import_merchant_data_bp = Blueprint('import_merchant_data_bp', __name__,
                     template_folder='templates',
                     static_folder='static',
                     url_prefix='/import-merchant-data')



logger = logging.getLogger('import')

import_merchant_data_api = Api(import_merchant_data_bp)

class TriggerPing(TriggerImportDataBaseResource):
    
    def get_task_url(self):
        return '/import-merchant-data/init-ping'
    
    def get_task_queue(self):
        return 'common'
    
class InitPing(InitImportDataBaseResource):
    pass    


class TriggerImportMerchantAcct(TriggerImportDataBaseResource):
    
    def get_task_url(self):
        return '/import-merchant-data/init-import-merchant-acct'
    
    def get_task_queue(self):
        return 'import-merchant'
    
    
    
    
class InitImportMerchantAcct(InitImportDataBaseResource):
    def get_import_data_count(self, **kwargs):
        count       = count_registered_merchant_function()
        return count
    
    def get_import_data_page_size(self):
        return analytics_conf.STREAM_DATA_PAGE_SIZE
    
    def get_task_url(self):
        return '/import-merchant-data/import-merchant-acct'
    
    def get_task_queue(self):
        return 'import-merchant'
    


class ImportMerchantAcct(ImportDataBaseResource): 
    def import_data(self, offset, limit, **kwargs):
        logger.debug('Going to import data now')
        logger.debug('offset=%d limit=%d', offset, limit)
        
        partition_month = datetime.now()
        try:
            start_cursor    = kwargs.get('start_cursor')
            batch_id        = kwargs.get('batch_id')
            
            logger.debug('ImportMerchantAcct: start_cursor=%s', start_cursor)
            logger.debug('ImportMerchantAcct: batch_id=%s', batch_id)
            
            next_cursor     = import_registered_merchant_function(dataset_name='system',  
                                                                      limit=limit, start_cursor=start_cursor, batch_id=batch_id)
            
            logger.debug('ImportAllRegisteredCustomer: next_cursor=%s', next_cursor)
            
            return next_cursor
        except:
            logger.debug('Failed due to %s', get_tracelog())
        
    
    def get_task_url(self):
        return '/import-merchant-data/import-merchant-acct'
    
    def get_task_queue(self):
        return 'import-merchant'

    
def import_registered_merchant_function(dataset_name='system', limit=analytics_conf.STREAM_DATA_PAGE_SIZE, start_cursor=None, batch_id=None):     
    
    bg_client       = create_bigquery_client(credential_filepath=analytics_conf.BIGQUERY_SERVICE_CREDENTIAL_PATH)
    db_client = create_db_client(caller_info="import_registered_merchant")
    merchant_acct_dict_list = []
    import_datetime         = datetime.now()
    with db_client.context():
        (merchant_acct_list, next_cursor) = MerchantAcct.list_all(limit = limit, start_cursor=start_cursor, return_with_cursor=True)
        
        
        for m in merchant_acct_list:
            merchant_acct_dict_list.append({
                                            'Key'                   : uuid.uuid1().hex,
                                            'MerchantKey'           : m.key_in_str,
                                            'CompanyName'           : m.company_name,
                                            'RegisteredDateTime'    : m.registered_datetime,
                                            'UpdatedDateTime'       : import_datetime,
                                            })
        
    
    merchant_data_dict_to_stream = {'':merchant_acct_dict_list}
    errors = []
    logger.debug('################################ batch_id= %s ############################################', batch_id)
    logger.debug(merchant_data_dict_to_stream)
    logger.debug('#####################################################################################################')
    
    errors = stream_data_by_datetime_partition(bg_client, dataset_name, REGISTERED_MERCHANT_TEMPLATE, 
                                               TABLE_SCHEME_TEMPLATE.get(REGISTERED_MERCHANT_TEMPLATE), merchant_data_dict_to_stream, 
                                               column_name_used_to_partition = 'RegisteredDateTime',
                                               partition_year=True
                         )
    
    if errors==[]:
        logger.debug("New rows have been added")
    else:
        logger.debug("Encountered errors while inserting rows: {}".format(errors))
        
    return next_cursor

def count_registered_merchant_function():
    db_client = create_db_client(caller_info="count_registered_merchant_function")
    with db_client.context():
        count = MerchantAcct.count(limit = MAX_FETCH_RECORD)
        
    return count

@import_merchant_data_bp.route('/count-merchant-acct')
def count_registered_customer():
    count = count_registered_merchant_function()
    return count, 200


import_merchant_data_api.add_resource(TriggerImportMerchantAcct,   '/trigger-import-merchant-acct')
import_merchant_data_api.add_resource(InitImportMerchantAcct,      '/init-import-merchant-acct')
import_merchant_data_api.add_resource(ImportMerchantAcct,          '/import-merchant-acct')

import_merchant_data_api.add_resource(TriggerPing,          '/trigger-ping')
import_merchant_data_api.add_resource(InitPing,          '/init-ping')





'''
Created on 30 Apr 2021

@author: jacklok
'''
from flask import Blueprint, render_template, session, abort, redirect, url_for, request
import logging, json, uuid
from flask.globals import current_app
from trexlib.utils.google.bigquery_util import create_bigquery_client, stream_data_by_datetime_partition
from trexanalytics import conf as analytics_conf
from trexanalytics.bigquery_table_template_config import CUSTOMER_TRANSACTION_TEMPLATE
from trexmodel.models.datastore.merchant_models import MerchantAcct
from trexmodel.models.datastore.customer_models import Customer
from trexmodel.utils.model.model_util import create_db_client 
from datetime import datetime
from trexlib.utils.log_util import get_tracelog
from trexanalytics.controllers.importdata.import_data_base_routes import TriggerImportDataBaseResource, InitImportDataBaseResource, ImportDataBaseResource
from flask_restful import Api
from trexlib.utils.string_util import is_not_empty, random_string
from trexanalytics.bigquery_table_template_config import TABLE_SCHEME_TEMPLATE
from trexmodel.models.datastore.transaction_models import CustomerTransaction
from trexanalytics.conf import MERCHANT_DATASET


import_customer_transaction_data_bp = Blueprint('import_customer_transaction_data_bp', __name__,
                     template_folder='templates',
                     static_folder='static',
                     url_prefix='/import-customer-transaction-data')

logger = logging.getLogger('import')

import_customer_transaction_data_api = Api(import_customer_transaction_data_bp)

class TriggerImportAllCustomerTransaction(TriggerImportDataBaseResource):
    
    def get_task_url(self):
        return '/import-customer-transaction-data/init-import-all-customer-transaction'
    
    def get_task_queue(self):
        return 'import-customer'
    
    
class InitImportAllCustomerTransaction(InitImportDataBaseResource):
    def get_import_data_count(self, **kwargs):
        count       = count_import_all_customer_transaction_function()
        #count = 1
        return count
    
    def get_import_data_page_size(self):
        return analytics_conf.STREAM_DATA_PAGE_SIZE
    
    def get_task_url(self):
        return '/import-customer-transaction-data/import-all-customer-transaction'
    
    def get_task_queue(self):
        return 'import-customer'
    


class ImportAllCustomerTransaction(ImportDataBaseResource): 
    def import_data(self, offset, limit, **kwargs):
        logger.debug('Going to import data now')
        logger.debug('offset=%d limit=%d', offset, limit)
        
        
        try:
            start_cursor    = kwargs.get('start_cursor')
            batch_id        = kwargs.get('batch_id')
            
            logger.debug('ImportAllCustomerTransaction: start_cursor=%s', start_cursor)
            
            next_cursor     = import_all_customer_transaction_function(dataset_name=MERCHANT_DATASET, limit=limit, start_cursor=start_cursor, batch_id=batch_id)
            
            logger.debug('ImportAllCustomerTransaction: next_cursor=%s', next_cursor)
            
            return next_cursor
        except:
            logger.debug('Failed due to %s', get_tracelog())
        
    
    def get_task_url(self):
        return '/import-customer-transaction-data/import-all-customer-transaction'
    
    def get_task_queue(self):
        return 'import-customer'
    
def count_import_all_customer_transaction_function():     
    
    credential_info = current_app.config['credential_config']
        
    db_client = create_db_client(info=credential_info, caller_info="count_import_all_customer_transaction_function")
    with db_client.context():
        count = CustomerTransaction.count()
        
    return count

def import_all_customer_transaction_function(credential_info=None, dataset_name=MERCHANT_DATASET, 
                                            limit=analytics_conf.STREAM_DATA_PAGE_SIZE, start_cursor=None, batch_id=None):     
    
    bg_client       = create_bigquery_client(credential_filepath=analytics_conf.BIGQUERY_SERVICE_CREDENTIAL_PATH)
    
    if credential_info is None:
        credential_info = current_app.config['credential_config']
        
    db_client                                   = create_db_client(info=credential_info, caller_info="import_all_customer_transaction_function")
    customer_transaction_data_dict_to_stream    = {}
    import_datetime                             = datetime.now()
    
    logger.debug('import_all_customer_transaction_function (%s): start_cursor=%s', batch_id, start_cursor)
    
    with db_client.context():
        (customer_transaction_list, next_cursor) = CustomerTransaction.list_all(limit = limit, start_cursor=start_cursor, return_with_cursor=True)
        import_count  = len(customer_transaction_list)
        
        for ct in customer_transaction_list:
            merchant_acct           = ct.transact_merchant_acct
            merchant_account_code   = merchant_acct.account_code.replace('-','')
            
            customer_transaction_dict_list = customer_transaction_data_dict_to_stream.get(merchant_account_code)
            
            if customer_transaction_dict_list is None:
                customer_transaction_dict_list = []
            
            customer_transaction_dict_list.append({
                                            "Key"                   : uuid.uuid1().hex,
                                            "UserKey"               : ct.transact_user_acct_key,
                                            "CustomerKey"           : ct.transact_customer_key,
                                            "MerchantKey"           : ct.transact_merchant_acct_key,
                                            "TransactOutlet"        : ct.transact_outlet_key,
                                            "TransactionId"         : ct.transaction_id,
                                            "InvoiceId"             : ct.invoice_id,
                                            "TransactAmount"        : ct.transact_amount,
                                            "TransactDateTime"      : ct.transact_datetime,
                                            'Reverted'              : False,
                                            'RevertedDateTime'      : None,
                                            "UpdatedDateTime"       : import_datetime,
                                            })
            
            
    
    
    
            customer_transaction_data_dict_to_stream = {merchant_account_code : customer_transaction_dict_list}
    
    logger.debug('################################ batch_id= %s ############################################', batch_id)
    logger.debug(customer_transaction_data_dict_to_stream)
    logger.debug('#####################################################################################################')
    
    errors = stream_data_by_datetime_partition(bg_client, dataset_name, CUSTOMER_TRANSACTION_TEMPLATE, 
                                               TABLE_SCHEME_TEMPLATE.get(CUSTOMER_TRANSACTION_TEMPLATE), customer_transaction_data_dict_to_stream, 
                                               column_name_used_to_partition='TransactDateTime', 
                                               partition_date=True,
                                               )
    
    if errors==[]:
        logger.debug("New rows have been added")
    else:
        logger.debug("Encountered errors while inserting rows: {}".format(errors))
    
    logger.debug('import_count=%d', import_count)
    
    return next_cursor

@import_customer_transaction_data_bp.route('/count-all-customer-transaction')
def count_customer_transaction():
    count = count_import_all_customer_transaction_function()
    return 'Total count=%s'% count


import_customer_transaction_data_api.add_resource(TriggerImportAllCustomerTransaction,   '/trigger-import-all-customer-transaction')
import_customer_transaction_data_api.add_resource(InitImportAllCustomerTransaction,   '/init-import-all-customer-transaction')
import_customer_transaction_data_api.add_resource(ImportAllCustomerTransaction,       '/import-all-customer-transaction')




'''
Created on 13 Jan 2021

@author: jacklok
'''
from flask import Blueprint, render_template, request, Response, current_app
from flask_restful import Resource
from trexlib.utils.google.cloud_tasks_util import create_task
from trexanalytics import conf as analytics_conf
import logging
from trexlib.utils.log_util import get_tracelog
from trexlib.utils.string_util import random_string

logger = logging.getLogger('import')

class TriggerImportDataBaseResource(Resource):
    
    def output_html(self, content, code=200, headers=None):
        resp = Response(content, mimetype='text/html', headers=headers)
        resp.status_code = code
        return resp
    
    def post(self):
        return self.get()
    
    def get(self):
        task_url = None
        try:
            task_url    = analytics_conf.UPSTREAM_BASE_URL + self.get_task_url()
            queue_name  = self.get_task_queue()
            payload     = self.get_data_payload()or {}
            
            logger.debug('task_url=%s', task_url)
            logger.debug('queue_name=%s', queue_name)
            logger.debug('payload=%s', payload)
            
            payload['batch_id'] = random_string(8, is_human_mistake_safe=True)
            
            create_task(task_url, queue_name, 
                        in_seconds      = self.get_task_delay_in_seconds(),
                        http_method     = self.get_task_http_method(), 
                        payload         = payload,
                        credential_path = analytics_conf.ANALYTICS_TASK_SERVICE_CREDENTIAL_PATH, 
                        project_id      = analytics_conf.ANALYTICS_TASK_GCLOUD_PROJECT_ID,
                        location        = analytics_conf.ANALYTICS_TASK_GCLOUD_LOCATION,
                        service_email   = analytics_conf.ANALYTICS_TASK_SERVICE_ACCOUNT_EMAIL
                        )
        except:
            logger.debug(get_tracelog())
        
        return self.output_html("Triggered task_url=%s"% (task_url))
    
    def get_task_url(self):
        pass
    
    def get_task_queue(self):
        pass
    
    def get_task_delay_in_seconds(self):
        return 1
    
    def get_task_http_method(self):
        return 'post'
        
    def get_data_payload(self):
        pass

class InitImportDataBaseResource(TriggerImportDataBaseResource):
    
    def post(self):
        return self.get()
    
    def get(self):
        content         = request.json or request.args or {}
        batch_id        = content.get('batch_id')
        
        data_payload    = self.get_data_payload() or {}
        task_url        = None
        
        logger.debug('InitImportDataBaseResource: data_payload=%s', data_payload)
        
        try:
            
            total_count     = self.get_import_data_count(**data_payload) or 0
            
            
            logger.debug('InitImportDataBaseResource: total_count=%s', total_count)
            
            if total_count>0:
                try:
                    page_size   = self.get_import_data_page_size()
                    task_count  = int(total_count/page_size)
                    remaining   = total_count % page_size
                    
                    if remaining>0:
                        task_count = task_count +1
                        
                        
                    
                    payload = {
                                'task_count'    : task_count,
                                'total_count'   : total_count,
                                'task_index'    : 1,
                                'page_size'     : page_size, 
                                'batch_id'      : batch_id,
                                }
                    
                    if data_payload is not None:
                        payload.update(data_payload)
                    
                    
                    task_url    = self.get_task_url()
                    if task_url:
                        task_url    = analytics_conf.UPSTREAM_BASE_URL + task_url
                        queue_name  = self.get_task_queue()
                        
                        logger.debug('InitImportDataBaseResource: task_url=%s', task_url)
                        logger.debug('InitImportDataBaseResource: queue_name=%s', queue_name)
                        
                        create_task(task_url, queue_name, 
                                    in_seconds      = self.get_task_delay_in_seconds(),
                                    http_method     = self.get_task_http_method(), 
                                    payload         = payload,
                                    credential_path = analytics_conf.ANALYTICS_TASK_SERVICE_CREDENTIAL_PATH, 
                                    project_id      = analytics_conf.ANALYTICS_TASK_GCLOUD_PROJECT_ID,
                                    location        = analytics_conf.ANALYTICS_TASK_GCLOUD_LOCATION,
                                    service_email   = analytics_conf.ANALYTICS_TASK_SERVICE_ACCOUNT_EMAIL
                                    )
                    
                        
                except:
                    logger.debug(get_tracelog())
                    return
        except:
            logger.error('Faield due to %s', get_tracelog()) 
        
        return self.output_html("Init %s" % task_url)
        
    
    def get_import_data_count(self, **kwargs):
        pass
    
    def get_import_data_page_size(self):
        pass
    
    def get_task_url(self):
        pass
    
    def get_task_queue(self):
        pass
    
    def get_task_http_method(self):
        return 'post'
    
class ImportDataBaseResource(InitImportDataBaseResource):    
    
    def get(self):
        content         = request.json or request.args or {}
        
        logger.debug('content=%s', content)
        logger.debug('***********************************************************')
        logger.debug('ImportDataBaseResource: content=%s', content)
        logger.debug('***********************************************************')
        
        
        task_count      = content.get('task_count')
        total_count     = content.get('total_count')
        task_index      = content.get('task_index')
        page_size       = content.get('page_size')
        start_cursor    = content.get('start_cursor')
        batch_id        = content.get('batch_id')
        
        data_payload    = self.get_data_payload() or {}
        
        data_payload.update(content)
        
        data_payload ['start_cursor'] = start_cursor 
        
        logger.debug('***********************************************************')
        logger.debug('ImportDataBaseResource: data_payload( %s- %d )=%s', batch_id, task_index, data_payload)
        logger.debug('***********************************************************')
        
        offset = 0
        if task_index>1:
            offset = (task_index-1) * page_size
        
        if task_index == task_count:
            #should complete here
            self.import_data(offset, page_size, **data_payload)
            
            logger.debug('Completed all import task')
            return self.output_html("Completed")
        
        elif task_index<task_count:
            #process import then pass to another task
            
            next_cursor         = self.import_data(offset, page_size, **data_payload)
            
            logger.debug('=========================== next_cursor=%s', next_cursor)
            
            existing_playload   = data_payload
            new_payload         = {
                                    'task_count'    : task_count,
                                    'total_count'   : total_count,
                                    'task_index'    : task_index + 1,
                                    'page_size'     : page_size, 
                                    'start_cursor'  : next_cursor,
                                    'batch_id'      : batch_id,
                                    }
            
            existing_playload.update(new_payload)
            
            logger.debug('>>>>>>>>>>>>>>>>>>>>>>>>>>> updated existing_playload=%s', existing_playload)
            
            task_url    = analytics_conf.UPSTREAM_BASE_URL + self.get_task_url()
            queue_name  = self.get_task_queue()
            
            logger.debug('ImportDataBaseResource: task_url=%s', task_url)
            logger.debug('ImportDataBaseResource: queue_name=%s', queue_name)
            
            create_task(task_url, queue_name, 
                        in_seconds      = self.get_task_delay_in_seconds(),
                        http_method     = self.get_task_http_method(), 
                        payload         = new_payload,
                        credential_path = analytics_conf.ANALYTICS_TASK_SERVICE_CREDENTIAL_PATH, 
                        project_id      = analytics_conf.ANALYTICS_TASK_GCLOUD_PROJECT_ID,
                        location        = analytics_conf.ANALYTICS_TASK_GCLOUD_LOCATION,
                        service_email   = analytics_conf.ANALYTICS_TASK_SERVICE_ACCOUNT_EMAIL
                        )
            
            logger.debug('Completed partial import task, pass to next task')
            
            return self.output_html("Pass to next task")
    
    
    
    def import_data(self, offset, limit, **kwargs):
        pass
        


'''
Created on 13 Jan 2021

@author: jacklok
'''

from flask import Blueprint, render_template, session, abort, redirect, url_for, request
import logging, json, uuid
from flask.globals import current_app
from trexlib.utils.google.bigquery_util import create_bigquery_client, stream_data_by_datetime_partition
from trexanalytics import conf as analytics_conf
from trexanalytics.bigquery_table_template_config import REGISTERED_CUSTOMER_TEMPLATE, MERCHANT_REGISTERED_CUSTOMER_TEMPLATE
from trexmodel.models.datastore.merchant_models import MerchantAcct
from trexmodel.models.datastore.customer_models import Customer
from trexmodel.utils.model.model_util import create_db_client 
from datetime import datetime
from trexlib.utils.log_util import get_tracelog
from trexanalytics.controllers.importdata.import_data_base_routes import TriggerImportDataBaseResource, InitImportDataBaseResource, ImportDataBaseResource
from flask_restful import Api
from trexlib.utils.string_util import is_not_empty, random_string
from trexanalytics.bigquery_table_template_config import TABLE_SCHEME_TEMPLATE


import_customer_data_bp = Blueprint('import_customer_data_bp', __name__,
                     template_folder='templates',
                     static_folder='static',
                     url_prefix='/import-customer-data')

logger = logging.getLogger('import')

import_customer_data_api = Api(import_customer_data_bp)


class TriggerImportAllRegisteredCustomer(TriggerImportDataBaseResource):
    
    def get_task_url(self):
        return '/import-customer-data/init-import-all-registered-customer'
    
    def get_task_queue(self):
        return 'import-customer'
    
    
    
    
class InitImportAllRegisteredCustomer(InitImportDataBaseResource):
    def get_import_data_count(self, **kwargs):
        count       = count_import_all_registered_customer_function()
        #count = 1
        return count
    
    def get_import_data_page_size(self):
        return analytics_conf.STREAM_DATA_PAGE_SIZE
    
    def get_task_url(self):
        return '/import-customer-data/import-all-registered-customer'
    
    def get_task_queue(self):
        return 'import-customer'
    


class ImportAllRegisteredCustomer(ImportDataBaseResource): 
    def import_data(self, offset, limit, **kwargs):
        logger.debug('Going to import data now')
        logger.debug('offset=%d limit=%d', offset, limit)
        
        
        try:
            start_cursor    = kwargs.get('start_cursor')
            batch_id        = kwargs.get('batch_id')
            
            logger.debug('ImportAllRegisteredCustomer: start_cursor=%s', start_cursor)
            
            next_cursor     = import_all_registered_customer_function(dataset_name='system', limit=limit, start_cursor=start_cursor, batch_id=batch_id)
            
            logger.debug('ImportAllRegisteredCustomer: next_cursor=%s', next_cursor)
            
            return next_cursor
        except:
            logger.debug('Failed due to %s', get_tracelog())
        
    
    def get_task_url(self):
        return '/import-customer-data/import-all-registered-customer'
    
    def get_task_queue(self):
        return 'import-customer'
    
def count_import_all_registered_customer_function():     
    
    credential_info = current_app.config['credential_config']
        
    db_client = create_db_client(info=credential_info, caller_info="count_import_all_registered_customer_function")
    with db_client.context():
        count = Customer.count()
        
    return count

def import_all_registered_customer_function(credential_info=None, dataset_name='system_dataset', 
                                            limit=analytics_conf.STREAM_DATA_PAGE_SIZE, start_cursor=None, batch_id=None):     
    
    bg_client       = create_bigquery_client(credential_filepath=analytics_conf.BIGQUERY_SERVICE_CREDENTIAL_PATH)
    
    if credential_info is None:
        credential_info = current_app.config['credential_config']
        
    db_client = create_db_client(info=credential_info, caller_info="import_registered_customer_function")
    customer_dict_list = []
    import_datetime         = datetime.now()
    
    logger.debug('import_all_registered_customer_function (%s): start_cursor=%s', batch_id, start_cursor)
    
    with db_client.context():
        (customer_list, next_cursor) = Customer.list_all(limit = limit, start_cursor=start_cursor, return_with_cursor=True)
        import_count  = len(customer_list)
        
        for c in customer_list:
            customer_dict_list.append({
                                            "Key"                   : uuid.uuid1().hex,
                                            "UserKey"               : c.registered_user_acct_key,
                                            "CustomerKey"           : c.key_in_str,
                                            "MerchantKey"           : c.registered_merchant_acct_key,
                                            "DOB"                   : c.birth_date,
                                            "Gender"                : c.gender,
                                            "MobilePhone"           : c.mobile_phone,
                                            "Email"                 : c.email,
                                            "MobileAppInstall"      : c.mobile_app_installed,
                                            "RegisteredDateTime"    : c.registered_datetime,
                                            "RegisteredOutlet"      : c.registered_outlet_key,
                                            "UpdatedDateTime"       : import_datetime,
                                            })
    
    customer_data_dict_to_stream = {'':customer_dict_list}
    
    logger.debug('################################ batch_id= %s ############################################', batch_id)
    logger.debug(customer_data_dict_to_stream)
    logger.debug('#####################################################################################################')
    
    errors = stream_data_by_datetime_partition(bg_client, dataset_name, REGISTERED_CUSTOMER_TEMPLATE, 
                                               TABLE_SCHEME_TEMPLATE.get(REGISTERED_CUSTOMER_TEMPLATE), customer_data_dict_to_stream, 
                                               column_name_used_to_partition='RegisteredDateTime', 
                                               partition_date=True,
                                               )
    
    if errors==[]:
        logger.debug("New rows have been added")
    else:
        logger.debug("Encountered errors while inserting rows: {}".format(errors))
    
    logger.debug('import_count=%d', import_count)
    
    return next_cursor

class TriggerImportMerchantRegisteredCustomer(TriggerImportDataBaseResource):
    
    def get_task_url(self):
        return '/import-customer-data/init-import-merchant-registered-customer'
    
    def get_task_queue(self):
        return 'import-customer'
    
    def get_data_payload(self):
        merchant_acct_key = request.args.get('merchant_acct_key')
        return {
                'merchant_acct_key': merchant_acct_key
                }
    
    
class InitImportMerchantRegisteredCustomer(InitImportDataBaseResource):
    def get_import_data_count(self, **kwargs):
        
        merchant_acct_key = kwargs.get('merchant_acct_key')
        
        logger.debug('InitImportMerchantRegisteredCustomer: merchant_acct_key=%s', merchant_acct_key)
        
        count       = count_import_merchant_registered_customer_function(merchant_acct_key)
        return count
    
    def get_import_data_page_size(self):
        return analytics_conf.STREAM_DATA_PAGE_SIZE
    
    def get_task_url(self):
        return '/import-customer-data/import-merchant-registered-customer'
    
    def get_task_queue(self):
        return 'import-customer'
    
    def get_data_payload(self):
        content             = request.json
        merchant_acct_key   = content.get('merchant_acct_key')
        
        return {
                'merchant_acct_key': merchant_acct_key
                }


class ImportMerchantRegisteredCustomer(ImportDataBaseResource): 
    def import_data(self, offset, limit, **kwargs):
        logger.debug('Going to import data now')
        logger.debug('offset=%d limit=%d', offset, limit)
        
        try:
            merchant_acct_key       = kwargs.get('merchant_acct_key')
            start_cursor            = kwargs.get('start_cursor')
            batch_id                = kwargs.get('batch_id')
            (errors, next_cursor)   = import_merchant_registered_customer_function(dataset_name='merchant', merchant_acct_key=merchant_acct_key,  
                                                         limit=limit, 
                                                         start_cursor=start_cursor, batch_id=batch_id)
        except:
            logger.debug('Failed due to %s', get_tracelog())
        
        return next_cursor
    
    def get_task_url(self):
        return '/import-customer-data/import-merchant-registered-customer'
    
    def get_task_queue(self):
        return 'import-customer'
    
    def get_data_payload(self):
        content             = request.json
        merchant_acct_key   = content.get('merchant_acct_key')
        
        return {
                'merchant_acct_key': merchant_acct_key
                }

def count_import_merchant_registered_customer_function(merchant_acct_key):     
    
    credential_info = current_app.config['credential_config']
        
    db_client = create_db_client(info=credential_info, caller_info="count_import_all_registered_customer_function")
    with db_client.context():
        merchant_acct = None
        if is_not_empty(merchant_acct_key):
            merchant_acct       = MerchantAcct.fetch(merchant_acct_key)
        count = Customer.count_merchant_customer(merchant_acct)
        
    return count


def import_merchant_registered_customer_function(credential_info=None, dataset_name='merchant', merchant_acct_key=None,
                                            partition_date = True, 
                                            limit=analytics_conf.STREAM_DATA_PAGE_SIZE, start_cursor=None, batch_id=None):     
    
    bg_client       = create_bigquery_client(credential_filepath=analytics_conf.BIGQUERY_SERVICE_CREDENTIAL_PATH)
    
    if credential_info is None:
        credential_info = current_app.config['credential_config']
        
    db_client = create_db_client(info=credential_info, caller_info="import_registered_customer_function")
    customer_list_dict      = {}
    import_datetime         = datetime.now()
    
    logger.debug('import_merchant_registered_customer_function: merchant_acct_key=%s', merchant_acct_key)
    
    with db_client.context():
        if is_not_empty(merchant_acct_key):
            merchant_acct = MerchantAcct.fetch(merchant_acct_key)
            (customer_list, next_cursor) = Customer.list_merchant_customer(merchant_acct, limit = limit, 
                                                                           start_cursor=start_cursor, return_with_cursor=True)
        else:
            customer_list = Customer.list_all(limit = limit, start_cursor=start_cursor)
        
        import_count  = len(customer_list)
        
        for c in customer_list:
            
            merchant_acct   = c.registered_merchant_acct
            account_code    = merchant_acct.account_code
            account_code    = account_code.replace('-','')
            
            customer_list = customer_list_dict.get(account_code)
            if customer_list is None:
                customer_list = []
            
            customer_list.append({
                                            "Key"                   : uuid.uuid1().hex,
                                            "UserKey"               : c.registered_user_acct_key,
                                            "CustomerKey"           : c.key_in_str,
                                            "DOB"                   : c.birth_date,
                                            "Gender"                : c.gender,
                                            "MobilePhone"           : c.mobile_phone,
                                            "Email"                 : c.email,
                                            "MobileAppInstall"      : c.mobile_app_installed,
                                            "RegisteredDateTime"    : c.registered_datetime,
                                            "RegisteredOutlet"      : c.registered_outlet_key,
                                            "UpdatedDateTime"       : import_datetime,
                                            })
            
            customer_list_dict[account_code] = customer_list
            
    logger.debug('################################ batch_id= %s ############################################', batch_id)
    logger.debug(customer_list_dict)
    logger.debug('#####################################################################################################')        
    
    errors = stream_data_by_datetime_partition(bg_client, dataset_name, MERCHANT_REGISTERED_CUSTOMER_TEMPLATE, 
                                               TABLE_SCHEME_TEMPLATE.get(MERCHANT_REGISTERED_CUSTOMER_TEMPLATE), customer_list_dict, 
                                               column_name_used_to_partition='RegisteredDateTime',partition_date=partition_date
                             )
    
    if errors==[]:
        logger.debug("New rows have been added")
    else:
        logger.debug("Encountered errors while inserting rows: {}".format(errors))
    
    logger.debug('import_count=%d', import_count)
    
    return (errors,next_cursor)

@import_customer_data_bp.route('/count-all-registered-customer')
def count_registered_customer():
    count = count_import_all_registered_customer_function()
    return count, 200

@import_customer_data_bp.route('/trigger-import-customer/<customer_key>')
def trigger_customer_changes(customer_key):
    
    import_merchant_customer_changes(customer_key)
    
    return 'Completed', 200
    
def import_merchant_customer_changes(customer_key, credential_info=None):
    bg_client       = create_bigquery_client(credential_filepath=analytics_conf.BIGQUERY_SERVICE_CREDENTIAL_PATH)
    
    if credential_info is None:
        credential_info = current_app.config['credential_config']
    
    db_client = create_db_client(info=credential_info, caller_info="import_registered_customer_function")
    merchant_customer_list_dict     = {}
    registered_customer_list_dict   = {}
    import_datetime                 = datetime.now()
    
    with db_client.context():
        customer = Customer.fetch(customer_key)
        merchant_acct   = customer.registered_merchant_acct
        account_code    = merchant_acct.account_code
        account_code    = account_code.replace('-','')
        registered_customer_list_dict[""] = [{
                                            "Key"                   : uuid.uuid1().hex,
                                            "UserKey"               : customer.registered_user_acct_key,
                                            "CustomerKey"           : customer.key_in_str,
                                            "MerchantKey"           : customer.registered_merchant_acct_key,
                                            "DOB"                   : customer.birth_date,
                                            "Gender"                : customer.gender,
                                            "MobilePhone"           : customer.mobile_phone,
                                            "Email"                 : customer.email,
                                            "MobileAppInstall"      : customer.mobile_app_installed,
                                            "RegisteredDateTime"    : customer.registered_datetime,
                                            "RegisteredOutlet"      : customer.registered_outlet_key,
                                            "UpdatedDateTime"       : import_datetime,
                                            }]
        merchant_customer_list_dict[account_code] = [{
                                            "Key"                   : uuid.uuid1().hex,
                                            "UserKey"               : customer.registered_user_acct_key,
                                            "CustomerKey"           : customer.key_in_str,
                                            "DOB"                   : customer.birth_date,
                                            "Gender"                : customer.gender,
                                            "MobilePhone"           : customer.mobile_phone,
                                            "Email"                 : customer.email,
                                            "MobileAppInstall"      : customer.mobile_app_installed,
                                            "RegisteredDateTime"    : customer.registered_datetime,
                                            "RegisteredOutlet"      : customer.registered_outlet_key,
                                            "UpdatedDateTime"       : import_datetime,
                                            }]
            
        
    
    logger.debug('#####################################################################################################')
    logger.debug(merchant_customer_list_dict)
    logger.debug('#####################################################################################################')        
    errors = []
    
    errors1 = stream_data_by_datetime_partition(bg_client, 'merchant', MERCHANT_REGISTERED_CUSTOMER_TEMPLATE, 
                                               TABLE_SCHEME_TEMPLATE.get(MERCHANT_REGISTERED_CUSTOMER_TEMPLATE), merchant_customer_list_dict, 
                                               column_name_used_to_partition='RegisteredDateTime', partition_date=True
                                               )
    
    errors2 = stream_data_by_datetime_partition(bg_client, 'system', REGISTERED_CUSTOMER_TEMPLATE, 
                                               TABLE_SCHEME_TEMPLATE.get(REGISTERED_CUSTOMER_TEMPLATE), registered_customer_list_dict, 
                                               column_name_used_to_partition='RegisteredDateTime', partition_date=True
                                               )
    errors.extend(errors1)
    errors.extend(errors2)
    
    if errors==[]:
        logger.debug("New rows have been added")
    else:
        logger.debug("Encountered errors while inserting rows: {}".format(errors))
    
    return errors


import_customer_data_api.add_resource(TriggerImportAllRegisteredCustomer,   '/trigger-import-all-registered-customer')
import_customer_data_api.add_resource(InitImportAllRegisteredCustomer,      '/init-import-all-registered-customer')
import_customer_data_api.add_resource(ImportAllRegisteredCustomer,          '/import-all-registered-customer')

import_customer_data_api.add_resource(TriggerImportMerchantRegisteredCustomer,   '/trigger-import-merchant-registered-customer')
import_customer_data_api.add_resource(InitImportMerchantRegisteredCustomer,      '/init-import-merchant-registered-customer')
import_customer_data_api.add_resource(ImportMerchantRegisteredCustomer,          '/import-merchant-registered-customer')





'''
Created on 15 Jan 2021

@author: jacklok
'''
import unittest, logging, sys
from trexanalytics.util.bigquery_util import detele_dataset

logger = logging.getLogger('unittest')

class DeleteBigqueryDatasetUnitTestCase(unittest.TestCase):
    def setUp(self):
        pass

        
    def tearDown(self):
        pass
    
    
    
    def test_delete_dataset(self):
        dataset_name    = 'unitest'
        deleted_dataset = detele_dataset(dataset_name)
        
        assert deleted_dataset is not None    
        
        
logging.basicConfig(stream=sys.stderr, level=logging.DEBUG)


import setuptools
from pip._vendor.pkg_resources import require

setuptools.setup(
     name='trex-analytic',  
     version='0.0.16',
     author="Jack Lok",
     author_email="sglok77@gmail.com",
     description="TRex analytics package",
     packages=setuptools.find_packages(),
     classifiers=[
         "Programming Language :: Python :: 3",
         "License :: OSI Approved :: MIT License",
         "Operating System :: OS Independent",
     ],
     install_requires=[            
          'flask==1.1.2',
          'Jinja2',
          'requests',
          'google-oauth',
          'google-cloud-bigquery',
          'google-cloud-tasks',
          'flask-restful',
          'webargs',
      ]
 )






'''
Created on 11 Jan 2021

@author: jacklok
'''

from flask import Blueprint, render_template, session, abort, redirect, url_for, jsonify
import logging, json
from flask.globals import current_app
from flask.json import jsonify
from json import JSONEncoder
from datetime import datetime, date
from trexanalytics import conf as analytics_conf
from google.oauth2 import service_account 
from google.cloud import bigquery

main_bp = Blueprint('main_bp', __name__,
                     template_folder='templates',
                     static_folder='static',
                     url_prefix='/')

logger = logging.getLogger('debug')

class DateTimeEncoder(JSONEncoder):
        #Override the default method
        def default(self, obj):
            if isinstance(obj, (date, datetime)):
                return obj.isoformat()


def create_bigquery_client(info=None, credential_filepath=None):
    if info:
        cred = service_account.Credentials.from_service_account_info(info)
        
    else:
        if credential_filepath:
            cred = service_account.Credentials.from_service_account_file(credential_filepath)
        else:
            cred = service_account.Credentials.from_service_account_file(
                                                            analytics_conf.BIGQUERY_SERVICE_CREDENTIAL_PATH)

    client = bigquery.Client(project=analytics_conf.GCLOUD_PROJECT_ID, credentials=cred)
    
    return client        

@main_bp.route('/')
def home_page(): 
    
    return 'Welcome to %s project' % analytics_conf.APPLICATION_NAME, 200
    
@main_bp.route('/config')
def app_config():     
    config_dict = {}
    config_dict.update(current_app.config)
    config_dict['BIGQUERY_SERVICE_CREDENTIAL_PATH'] = analytics_conf.BIGQUERY_SERVICE_CREDENTIAL_PATH
    
    config_json_str = json.dumps(config_dict, indent=4, cls=DateTimeEncoder)
    #return jsonify(json.dumps(config_dict, indent=4, cls=DateTimeEncoder))
    
    response = current_app.response_class(
        response=config_json_str,
        status=200,
        mimetype='application/json'
    )
    return response

@main_bp.route('/ping')
def ping():
    return 'ping', 200

@main_bp.route('/test')
def test():
    bigquery_client = create_bigquery_client()
    
    logger.debug('bigquery_client=%s', bigquery_client)
    
    return 'Test', 200


